/*
ltop16outputset.c

Code created with "Geany" IDE on Puppy Linux

Compiled with: gcc -Wall -c "%f" top16linuxlibrary.c
Built with: gcc -Wall -ldl "%f" top16linuxlibrary.o 

29-06-2008	version 1.0 created



*/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <time.h>
#include <dlfcn.h>
#include <unistd.h>

#include "top16linuxlibrary.h"



//#define DEBUG (1) 


#define true TRUE
#define false FALSE

#define VERSIONSTRING ("version 1.1") 

void printHelpMessage()
{	
	printf("TCTEC top16 output command for linux\n") ;
	printf("%s\n",VERSIONSTRING) ;
	printf("www.tctec.net\n") ;
	printf("\n-------------------------------\n") ;
	printf("Commands:\n") ;
	printf("\n-l\t\tList all connected top16 modules\n") ;
	printf("\nUSAGE:\n") ;
	printf("ltop16outputset [-u,-s][SERIAL NUMBER],[MASK][ -tTIME IN MILLISECONDS]\n") ;
	printf("\t\t\t\t-t :optional for momentary action (1 to 10000)\n") ;
	printf("\t\t\t\t-u :{u}nset the output/s\n") ;
	printf("\t\t\t\t-s :{s}et the output/s\n") ;
	printf("\t\t\t\tMASK is a number representing bits (01 to 255)\n\n") ;
	printf("-s[SERIAL NUMBER],[MASK]\tSet the output of a module (Turn ON)\n") ;
	printf("\t\t\t\te.g: -sFTQ57KP1,255\n") ;
	printf("\t\t\t( 255 = binary 11111111. Therefore all outputs on. ) ") ;
	printf("\n\t\t\te.g: -uFTQ57KP1,255\n") ;
	printf("\t\t\t( 255 = binary 11111111. Therefore all outputs off. ) ") ;
	printf("\n\n\n") ;		
	
}

int main(int argc, char** argv) 
{		
	char serialNum[255] ="";
	char* serNumEnd ;
	int state ;
	uint16_t momentaryPause = 0 ;
	int32_t on = 0 ;  // (1 == on, 0 == off) 
	
	char boards [10][20] ;  // upto 10 boards could be connect in this case
	int32_t numberOfBoards ;
	int tempi ;
	int32_t Handle ;
	
		
			
	if (argc > 1)
	{
		openLibrary() ;  // open the ftdi library
		
		// parse arguments
		//
		if (strcmp(argv[1], "-l") == 0)
		{
			numberOfBoards = ListTop16Boards(boards) ;
			printf("Number of Top16 boards found = %d\n", numberOfBoards) ;
			printf("---------------------------------\n") ;
			for(tempi = 0 ; tempi < numberOfBoards; tempi++) 
			{
				printf("%s\n", boards[tempi]) ;				
			}						
		}
		else if ((strncmp(argv[1], "-s", 2) == 0) || (strncmp(argv[1], "-u", 2) == 0))
		{
			
			#ifdef DEBUG
			printf("Setting outputs\n") ;
			#endif
			
			state = -1 ;
			
			if (strncmp(argv[1], "-s", 2) == 0)
			{
				on = 1;
			}
			else
			{
				on = 0 ;
			}
			
			
			// find the serial number in the argument
			//
		
		    // ignore the "-s" or "-u"			
		    //
			strncpy(serialNum, (char*)&argv[1][2], 250) ;  
			
			serNumEnd = memchr(serialNum, ',', strlen(serialNum)) ; // find the comma
			if (serNumEnd)
			{				
				// get the state value from the string
				//
				state = atoi(serNumEnd+1) ;
			
				(*serNumEnd) = (char) NULL ;  // null terminate the serial number string
			}
			
			#ifdef DEBUG
			printf("SerialNumber = %s\n", serialNum) ;
			printf("State = %d\n", state) ;
			#endif
			
			// validate the state value
			//
			if ((state < 0) || (state > 255))
			{
				printf("Invalid state value.\n") ;
				
			}
			
			// have we specified a momentary action ?
			//			
			if (argc >= 3)
			{
				if (strncmp(argv[2], "-t", 2) == 0)
				{
					momentaryPause = atoi((char*)&argv[2][2]) ;
					
					// must be less than or equal to 10 seconds
					//
					if (momentaryPause > 10000)
					{
						momentaryPause = 0 ;
					}
					
					#ifdef DEBUG
					printf("Momentary Pause selected: %d milli seconds.\n", momentaryPause) ;
					#endif
				}				
			}
			Handle = OpenBoard(serialNum) ;			
			if (Handle > 0)
			{
				SetOutputs(Handle, state, on) ;
				if (momentaryPause > 0)
			    {
				   usleep(momentaryPause*1000);
				
				   // do the opposite to above, ie if relays on, then turn off
				   //
				   SetOutputs(Handle, state, ((~on) & 0x01)) ;				   
			    }
			    CloseBoard(Handle) ;  // close the board			    
			}
			
									
		}
		else 
		{
			printHelpMessage() ;
		}		
		
		closeLibrary() ;  // -- must be done
	}
	else   // no arguments specified, so print help message
	{
		printHelpMessage() ;
	}	
	exit(0) ;	
		
}

