/* 
top16linuxlibrary.c

Project: top16 linux library 
Always increment the version (in version function) when changes made


18-06-2008            Version 1.0            Created (c) TCTEC Pty Ltd, all rights reserved
15-03-2009			  Version 1.1            Removed util.h
13-Sep-2010           Version 1.2            Added ReadPulseCount function

*/
 


#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <time.h>
#include <dlfcn.h>
#include "top16linuxlibrary.h"
//#include "util.h"


/* ----------- FTDI library stuff -------------- */
void *FunctionLib;		/* handle to shared lib file */
uint32_t (*Function)();		/* Pointer to loaded routine */
const char *dlError;	/* Pointer to error string */

// function pointers into ftd2xx dll
//
uint32_t (*fpFT_CreateDeviceInfoList) ();
uint32_t (*fpFT_GetDeviceInfoDetail) () ;
uint32_t (*fpFT_Close) () ;
uint32_t (*fpFT_OpenEx) () ;
uint32_t (*fpFT_SetFlowControl) () ;
uint32_t (*fpFT_SetDataCharacteristics) () ;
uint32_t (*fpFT_SetBaudRate) () ;
uint32_t (*fpFT_SetTimeouts) () ;
uint32_t (*fpFT_Purge) () ;
uint32_t (*fpFT_Write) () ;
uint32_t (*fpFT_Read) () ;


//#define FTDIDLL "FTD2XX" 
#define true (1)
#define false (0)
#define TRUE (1) 
#define FALSE (0)

#define BOOL (int) 



// FTDI constants
//
const uint32_t FT_LIST_NUMBER_ONLY =	0x80000000 ;
const uint32_t FT_LIST_BY_INDEX =		0x40000000 ;
const uint32_t FT_LIST_ALL = 			0x20000000 ;

const uint32_t FT_OPEN_BY_SERIAL_NUMBER =	1 ;
const uint32_t FT_OPEN_BY_DESCRIPTION =		2 ;
const uint32_t FT_OPEN_BY_LOCATION = 		4 ;

const uint32_t FT_OK = 0 ;
const uint8_t FT_BITS_8 = 8 ;
const uint8_t FT_STOP_BITS_1 = 0 ;
const uint8_t FT_PARITY_NONE = 0 ;
const uint16_t FT_FLOW_NONE = 0x0000 ;

const uint32_t FT_BAUD_115200 = 115200 ; 
const uint32_t FT_BAUD_230400 = 230400 ;
const uint32_t FT_PURGE_RX = 1 ;
const uint32_t FT_PURGE_TX = 1 ;



/*---------------------- Utility Functions ---------------------------*/

/*
Converts a human readable character to a number representation.

Returns <0 if error

*/

int hexcharToNibble(char hex)
{
  int output ;
  
  output = 0 ;
  
  if ((hex >= '0') && (hex <= '9'))
  {
    output |= (hex - '0') ;
  }
  else if ((hex >= 'A') && (hex <= 'F'))
  {
    output = 0x0A + (hex - 'A') ;
  }
  else
  {
    output = -1 ;
  }  
      
  return(output) ;
}


//------------------- DL (Dynamic Library) Functions -----------------------

void loadDLFunction(char* funcName)
{
	Function = dlsym(FunctionLib, funcName) ;
	dlError = dlerror() ;
	#ifdef	 DEBUG	
	printf("loaded %s, return = %s\n", funcName, dlError) ;
	#endif
	if (dlError) exit(1) ;	
	
}

void openLibrary()
{	
	// Open Dynamic Loadable Library with absolute path
	//
	//FunctionLib = dlopen("/mnt/home/Super4/libftd2xx.so.0.4.13", RTLD_LAZY) ;
	//dlError = dlerror() ;
	
	FunctionLib = dlopen ("libftd2xx.so", RTLD_LAZY) ;
	dlError = dlerror() ;
	
	#ifdef DEBUG
	printf("Open Library with absolute path return = %s\n", dlError) ;
	#endif
	
	if (dlError) 
	{
		printf("Could not load library libftd2xx.so\n") ;
		exit(1) ;	
	}	
	
	// load the functions
	//
	loadDLFunction("FT_CreateDeviceInfoList") ;
	fpFT_CreateDeviceInfoList = Function ;
	
	loadDLFunction("FT_GetDeviceInfoDetail") ;
	fpFT_GetDeviceInfoDetail = Function ;
	
	loadDLFunction("FT_Close") ;
	fpFT_Close = Function ;
	
	loadDLFunction("FT_OpenEx") ;
	fpFT_OpenEx = Function ;
	
	loadDLFunction("FT_SetFlowControl") ; 
	fpFT_SetFlowControl = Function ;
	
	loadDLFunction("FT_SetDataCharacteristics") ; 
	fpFT_SetDataCharacteristics = Function ;
	
	loadDLFunction("FT_SetBaudRate") ;
	fpFT_SetBaudRate = Function ; 
	
	loadDLFunction("FT_SetTimeouts") ;
	fpFT_SetTimeouts = Function ;
	
	loadDLFunction("FT_Purge") ;
	fpFT_Purge = Function ;
	
	loadDLFunction("FT_Write") ;
	fpFT_Write = Function ;
	
	loadDLFunction("FT_Read") ;
	fpFT_Read = Function ;
}

void closeLibrary()
{
	int rc ;
	// Close the shared library
	//
	rc = dlclose(FunctionLib) ;
	dlError = dlerror() ;
	
	#ifdef DEBUG
	printf("Close function lib returned %s\n", dlError) ;	
	#endif	
}



void debug(char* string)
{
     printf("%s\n", string) ;           
}

/*
Returns the version of this DLL.

Returns:
        major and minor version number.
        i.e
        11 = version 1.1
        24 = version 2.4
        etc
*/

int32_t dllversion()
{
		printf("Dll version function.\n") ;
          return(12) ;
}

/*
Search for attached TCTEC Top16 DIO boards.
The stringArray will be filled with the names of the boards.
It must point to a 2 dimensional char (8bit) array of dimensions [n][20]
where n is the number of boards expected.

Returns: 
         The number of top16 boards found or 0 if none found.

*/
int32_t ListTop16Boards(char stringArray[][20])
{
    int32_t numDevices ;
    uint32_t temp ;          
    uint32_t Flags;
    uint32_t deviceType;
    uint32_t id ;
    uint32_t locID ;
    char pcSerialNumber[100] ;
    char pcDescription [200] ;
    uint32_t Handle ;  
    
    int32_t deviceCount = 0 ;  

    // get details of connected FTDI devices
	//		
	(*fpFT_CreateDeviceInfoList)  (&numDevices) ;	

	// list all the devices with the correct description
	//	
	for(temp = 0 ; temp < numDevices; temp++)
	{
		
		(*fpFT_GetDeviceInfoDetail) (temp, &Flags, &deviceType, &id, &locID, pcSerialNumber, pcDescription, &Handle) ;
           
        if (strstr(pcDescription, "TCTEC TOP16DIO") != NULL)        
        {
            // found one
            //      
            strncpy(stringArray[deviceCount], pcSerialNumber, 19) ;
            deviceCount += 1;
        }
    
    }
      
    return(deviceCount) ;    
}

/*
Close an open board.

Handle = Handle of open board.

Returns:
0 == success 

*/

int32_t CloseBoard(uint32_t Handle)
{
  
  return (int32_t) (*fpFT_Close) (Handle) ;
  
}

/*
Initialize a top16 board, returns a Handle (uint32)to opened board if successful.
Must be called before set and get commands.

Returns:
 >0  = Handle of opened board.
 -1 = invalid board
 

*/
uint32_t OpenBoard(char* board)
{    
    uint32_t tempInt ;
	uint32_t Handle ;		
		
	// open a device 
	//				
	tempInt = (*fpFT_OpenEx) ((void*)board, FT_OPEN_BY_SERIAL_NUMBER, &Handle) ;
	if (tempInt != FT_OK)
	{		
		return(-1) ;
	}		
	
    // set the baud rate, data format and flow control of the FTDI chip
    //    
    tempInt = (*fpFT_SetFlowControl) (Handle, FT_FLOW_NONE, 0, 0) ;
            
    tempInt += (*fpFT_SetDataCharacteristics) (Handle, FT_BITS_8, FT_STOP_BITS_1, FT_PARITY_NONE) ;
       
    tempInt += (*fpFT_SetBaudRate) (Handle, FT_BAUD_115200) ;
         
    tempInt += (*fpFT_SetTimeouts) (Handle, 2000, 500) ;     
    
    tempInt += (*fpFT_Purge) (Handle, FT_PURGE_RX | FT_PURGE_TX) ; 
         
    if (tempInt != FT_OK)
    {    	        
                (*fpFT_Close) (Handle) ;
                return (-1) ;
    }     
        
    return(Handle) ;
}

/*
Set the digital output states.
8 bits of the 'mask' determines which outputs will be affected:
    1 = output will be affected
    0 = leave alone (don't change)
    
'set' determines whether the output will be switched on or switched off.
1 = output on (pulled low)
0 = output off (not pulled low). 

Returns:
        The state of the outputs after the action.
        0 to FF:
        1 = output is on (pulled low).
        0 = output is off (not pulled low)
        
        A negative number indicates an error:
                   
        -1 invalid board handle (board not found)
        -2 Library function not loaded (FTD2XX library not found)
        -3 Invalid bit mask (must be 0 to FF)
        -4 Undefined error 

*/
int32_t SetOutputs(uint32_t Handle, uint8_t mask, uint8_t set)
{
          
    uint32_t tempInt ;	
	uint32_t byteCount ;
	uint32_t outputState ;
	int retry = TRUE ;
	int retryCount = 3 ;
		
	char rxCommand[10] ;
	char txCommand[10] ;
		
	
	// Send command
	//
	if (set)
	{
	  sprintf(txCommand, "#FF%1X%1X\r", ((mask & 0xF0)>>4), (mask & 0x0F)) ; 
    }else
    {
       sprintf(txCommand, "#00%1X%1X\r", ((mask & 0xF0)>>4), (mask & 0x0F)) ;    
    }	
	
    // ---------------- send loop -------------------
    while (retry && (retryCount > 0))
    {	
      retry = FALSE ;
      retryCount-- ;
            
      tempInt = (*fpFT_Write) (Handle, (void*) &txCommand, 6, &byteCount) ;      
      
      if (tempInt != FT_OK)    
      {       
        return (-4) ;
      }    
          
      tempInt = (*fpFT_Read) (Handle, (void*) &rxCommand, 6, &byteCount) ;
               
      if (tempInt != FT_OK)
      {        
        return (-4) ;             
      }
    
      if (byteCount >= 6)
      {
        // returned data is in the format >IIOO[CR]
        //                 
        if (rxCommand[0] != '>')   // validate response
        {          
         retry = true ;                      
        }else
        {
          // decode response
          //
          outputState = hexcharToNibble(rxCommand[3]) ;
          outputState <<= 4 ;
          outputState |= hexcharToNibble(rxCommand[4]) ;   
          
          if ((outputState > 0xFF) || (outputState < 0))
          {
            retry = true ;
          }
        }      
        
      }else
      {
        retry = true ;
      }
      
      if (retry == true)
      {
        // purge the RX and TX buffer
        //        
        (*fpFT_Purge) (Handle, (FT_PURGE_RX | FT_PURGE_TX)) ;
      }
    }   
    // --------------------- end send loop ----------------------- 
        
    if (retry == true)
    {
       // tried too many times
       //                   
       return (-4) ;
    }    
	    	
	return(outputState) ;          
}


/*

Read the digital inputs.

Returns:
        The state of the digital inputs
        0 to FF:
        1 = input is high
        0 = input is low
        
        A negative number indicates an error:
                   
        -1 invalid board handle (board not found)
        -2 DLL function not loaded (FTD2XX library not found)
        -4 Undefined error 

*/
int32_t GetInputs(uint32_t Handle)
{
          
    uint32_t tempInt ;	
	uint32_t inputState ;
	int retry = TRUE ;
	int retryCount = 3 ;
	uint32_t byteCount ;
			
	char rxCommand[10] ;
	char txCommand[10] ;
		
	
	// Send dummy set command
	//
    sprintf(txCommand, "#SSSS\r") ; 
    
	
    // ---------------- send loop -------------------
    while (retry && (retryCount > 0))
    {	
      retry = FALSE ;
      retryCount-- ;
            
      tempInt = (*fpFT_Write) (Handle, (void*) &txCommand, 6, &byteCount) ;      
      
      if (tempInt != FT_OK)    
      {       
        return (-4) ;
      }    
          
      tempInt = (*fpFT_Read) (Handle, (void*) &rxCommand, 6, &byteCount) ;
    
           
      if (tempInt != FT_OK)
      {        
        return (-4) ;             
      }
    
      if (byteCount >= 6)
      {
        // returned data is in the format >IIOO[CR]
        //                 
        if (rxCommand[0] != '>')   // validate response
        {          
         retry = true ;                      
        }else
        {
          // decode response
          //
          inputState = hexcharToNibble(rxCommand[1]) ;
          inputState <<= 4 ;
          inputState |= hexcharToNibble(rxCommand[2]) ;   
          
          if ((inputState > 0xFF) || (inputState < 0))
          {
            // the value is invalid, something went wrong
            //              
            retry = true ;
          }
        }      
        
      }else
      {
        retry = true ;
      }
      
      if (retry == true)
      {
        // purge the RX and TX buffer
        //        
        (*fpFT_Purge) (Handle, (FT_PURGE_RX | FT_PURGE_TX)) ;
      }
    }   
    // --------------------- end send loop ----------------------- 
            
    if (retry == true)
    {
       // tried too many times
       //                   
       return (-4) ;
    }    
	    	
	return(inputState) ;          
}

/*
Read an analog input.
The internal analog amplifier is set to the gain specified, then the input is sampled.

Handle  A Handle of an open Top16 board.
input   1 to 8
gain:
     'Z' = 1
     'Y' = 2
     'X' = 4
     'W' = 8
     'V' = 16
     'U' = 32
     'T' = 64

Returns:
       The raw analog input value (12 bit : 0 to 4096)
       at the gain specified.
       
        
        A negative number indicates an error:
                   
        -1 invalid board handle (board not found)
        -2 DLL function not loaded (FTD2XX.DLL not found)
        -3 invalid gain
        -4 Undefined error 

*/
uint32_t readAnalogInput(uint32_t Handle, int16_t input, uint8_t gain)
{
          
    uint32_t tempInt ;	
	uint32_t inputState = -4;
	int retry = TRUE ;
	int retryCount = 3 ;
	uint32_t byteCount ;
			
	char rxCommand[10] ;
	char txCommand[10] ;
		
	
	if ((input < 1) || (input > 8))
	{
       return (-4) ;       
    }
    if ((gain > 'Z') || (gain < 'T'))
    {
      return (-3) ; // invalid gain
    }
    
	// Send command
	//
    sprintf(txCommand, "#%c%1XZZ\r", gain, input) ; 
    
	
    // ---------------- send loop -------------------
    while (retry && (retryCount > 0))
    {	
      retry = FALSE ;
      retryCount-- ;
            
      tempInt = (*fpFT_Write) (Handle, (void*) &txCommand, 6, &byteCount) ;      
      
      if (tempInt != FT_OK)    
      {          	
        return (-4) ;
      }              
      tempInt = (*fpFT_Read) (Handle, (void*) &rxCommand, 6, &byteCount) ;
    
           
      if (tempInt != FT_OK)
      {       
      	 return (-4) ;             
      }    
      
      if (byteCount >= 6)
      {
        // returned data is in the format >nnnn[CR]
        //                 
        if (rxCommand[0] != '>')   // validate response
        {          
         retry = true ;                      
        }else
        {        
          // decode response
          //
          inputState = hexcharToNibble(rxCommand[1]) ;
          inputState <<= 4 ;
          inputState |= hexcharToNibble(rxCommand[2]) ;
          inputState <<= 4 ;
          inputState |= hexcharToNibble(rxCommand[3]) ;   
          inputState <<= 4 ;
          inputState |= hexcharToNibble(rxCommand[4]) ;               
          
        }      
        
      }else
      {
        retry = true ;
      }
      
      if (retry == true)
      {
        // purge the RX and TX buffer
        //        
        (*fpFT_Purge) (Handle, (FT_PURGE_RX | FT_PURGE_TX)) ;
      }
    }   
    // --------------------- end send loop ----------------------- 
    
    //(lpfnFT_Close) (Handle) ;
    
    if (retry == true)
    {
       // tried too many times
       // 
                  
       return (-4) ;
    }    
	    	
	return(inputState) ;          
}

/*
Get the pulse count of a particular input.
The pulse count is reset to zero automatically after read.

Returns:

0 to 32767

*/


uint32_t ReadPulseCount(uint32_t Handle, int16_t input)
{
          
    uint32_t tempInt ;	
	uint32_t inputState = -4;
	int retry = TRUE ;
	int retryCount = 3 ;
	uint32_t byteCount ;
			
	char rxCommand[10] ;
	char txCommand[10] ;
		
	
	if ((input < 1) || (input > 8))
	{
       return (-4) ;       
    }
   
    
	// Send command
	//
    sprintf(txCommand, "#I%1X\r", gain, input) ; 
    
	
    // ---------------- send loop -------------------
    while (retry && (retryCount > 0))
    {	
      retry = FALSE ;
      retryCount-- ;
            
      tempInt = (*fpFT_Write) (Handle, (void*) &txCommand, 4, &byteCount) ;      
      
      if (tempInt != FT_OK)    
      {          	
        return (-4) ;
      }              
      tempInt = (*fpFT_Read) (Handle, (void*) &rxCommand, 6, &byteCount) ;
    
           
      if (tempInt != FT_OK)
      {       
      	 return (-4) ;             
      }    
      
      if (byteCount >= 6)
      {
        // returned data is in the format >nnnn[CR]
        //                 
        if (rxCommand[0] != '>')   // validate response
        {          
         retry = true ;                      
        }else
        {        
          // decode response
          //
          inputState = hexcharToNibble(rxCommand[1]) ;
          inputState <<= 4 ;
          inputState |= hexcharToNibble(rxCommand[2]) ;
          inputState <<= 4 ;
          inputState |= hexcharToNibble(rxCommand[3]) ;   
          inputState <<= 4 ;
          inputState |= hexcharToNibble(rxCommand[4]) ;               
          
        }      
        
      }else
      {
        retry = true ;
      }
      
      if (retry == true)
      {
        // purge the RX and TX buffer
        //        
        (*fpFT_Purge) (Handle, (FT_PURGE_RX | FT_PURGE_TX)) ;
      }
    }   
    // --------------------- end send loop ----------------------- 
    
    //(lpfnFT_Close) (Handle) ;
    
    if (retry == true)
    {
       // tried too many times
       // 
                  
       return (-4) ;
    }    
	    	
	return(inputState) ;          
}


/*
Set a PWM output

	pwm		pwm setting 0 to 255
	output	1 to 8
	Handle	the handle of an open top16 board

Returns:

	0 Success

	-2 invalid output
	-4 Communication erro
	

*/

int32_t SetPWM(uint32_t Handle, uint8_t pwm, uint8_t output)
{
 	uint32_t tempInt ;	
	uint32_t byteCount ;	
	int retry = TRUE ;
	int retryCount = 3 ;
		
	char rxCommand[10] ;
	char txCommand[10] ;
		
	
	if ((output <= 0) || (output > 8))
	{
		  return(-2) ;
	}
	
	// Send command
	//
	if (pwm <= 0x0F)
	{
	  sprintf(txCommand, "#P%1X0%1X\r", output, pwm) ; 
    }else
    {
      sprintf(txCommand, "#P%1X%2X\r", output, pwm) ;    
    }	
	
    // ---------------- send loop -------------------
    while (retry && (retryCount > 0))
    {	
      retry = FALSE ;
      retryCount-- ;
            
      tempInt = (*fpFT_Write) (Handle, (void*) &txCommand, 6, &byteCount) ;      
      
      if (tempInt != FT_OK)    
      {       
        return (-4) ;
      }    
          
      tempInt = (*fpFT_Read) (Handle, (void*) &rxCommand, 4, &byteCount) ;              
      
      if (tempInt != FT_OK)
      {        
        return (-4) ;             
      }
    
      if (byteCount >= 4)
      {
        // returned data is in the format >IIOO[CR]
        //                 
        if (rxCommand[0] != '>')   // validate response
        {          
         retry = true ;                      
        }else
        {
            // do nothing
        }      
        
      }else
      {
        retry = true ;
      }
      
      if (retry == true)
      {
        // purge the RX and TX buffer
        //        
        (*fpFT_Purge) (Handle, (FT_PURGE_RX | FT_PURGE_TX)) ;
      }
    }   
    // --------------------- end send loop ----------------------- 
        
    if (retry == true)
    {
       // tried too many times
       //                   
       return (-4) ;
    }    
	    	
	return(0) ;          
}
